/* this example code shows how to switch the 8051 development board's
 * serial port mux between the two ports, using a different baud rate
 * on each one.  This examples works with polled I/O.  A different
 * approach must be used if interrupt driven I/O is used.
 */


#include "delay_ms.h"
#include "paulmon2.h"
#include <stdio.h>
#include <8051.h>

#define SERIAL_PORT_MUX_PIN (P3_4)

/* these settings are for timer1 and a 22.1184 MHz cyrstal */
#define TH1_BAUD_115200  255
#define TH1_BAUD_57600   254
#define TH1_BAUD_38400   253
#define TH1_BAUD_19200   250
#define TH1_BAUD_9600    244
#define TH1_BAUD_2400    208
#define TH1_BAUD_1200    160

/* function declarations */
void switch_to_aux_port(void);
void switch_to_main_port(void);
void putchar(char c);


/* this variable remembers the original baud rate that was */
/* established (by PAULMON2) when we started running */
unsigned char orig_baud;


/* this simple main program sends messages to the two serial */
/* ports, each at its own baud rate. */

void main()
{
	unsigned int aux_count=1, main_count=1;

	printf_fast("Serial port mux example program: prints messages\r\n");
	printf_fast("to both serial ports, each at its own baud rate.\r\n");
	printf_fast("One message will appear on this main port for\r\n");
	printf_fast("every 12 messages sent to the aux port.\r\n");

	orig_baud = TH1;
	switch_to_aux_port();

	while (1) {
		printf_fast("message #%u on aux serial port\r\n", aux_count);
		aux_count++;
		if (aux_count % 12 == 0) {
			switch_to_main_port();
			printf_fast("msg #%u on program port\r\n", main_count);
			main_count++;
			switch_to_aux_port();
		}
		delay_ms(140);
	}
}


/* these two functions show the most efficient way to switch */
/* between the two ports with only minimal delay.  This approach */
/* works when polled I/O is used.  A different (and more complex) */
/* approach would be needed in interrupt driven I/O is used */

void switch_to_aux_port(void)
{
	while (!TI) ;                   /* wait for UART transmit to be idle */
	SERIAL_PORT_MUX_PIN = 0;        /* switch to the aux serial port */
	TH1 = TH1_BAUD_2400;		/* set the baud rate */
	TF1 = 0;
	while (!TF1) ;			/* wait for timer1, 1 new bit time */
}

void switch_to_main_port(void)
{
	while (!TI) ;                   /* wait for UART transmit to be idle */
	SERIAL_PORT_MUX_PIN = 1;        /* switch to the main serial port */
	TH1 = orig_baud;		/* set the original baud rate */
	TF1 = 0;
	while (!TF1) ;			/* wait for timer1, 1 new bit time */
}



/* required for using printf, printf_large, printf_fast, etc */
/* the printf routines call to putchar to actually send output */

void putchar(char c)
{
        pm2_cout(c);
}



